// --------------------------------
// <copyright file="NameValueCombination.cs" company="TRONSoft">
//     Copyright (c) TRONSoft.
// </copyright>
// <license>
//   This source code is subject to terms and conditions of the Microsoft
//   Public License. A copy of the license can be found in the License.html
//   file at the root of this distribution. If you cannot locate the
//   Microsoft Public License, please send an email to dlr@microsoft.com.
//   By using this source code in any fashion, you are agreeing to be bound 
//   by the terms of the Microsoft Public License. You must not remove this
//   notice, or any other, from this software.
// </license>
// <author>Tron</author>
// ---------------------------------

using System;

namespace Tronsoft.Actucal.CalculationEngineCreator
{
    [Serializable]
    public class NameValueCombination<T> : INameValueCombination where T : IConvertible
    {
        private readonly string name;
        private T value;

        public NameValueCombination(string name) : this(name, default(T))
        {
        }

        public NameValueCombination(string name, T value)
        {
            if (string.IsNullOrEmpty(name))
                throw new ArgumentNullException("name");
            if (name.Contains(" "))
                throw new ArgumentException("A name cannot contains spaces.", "name");
           
            this.name = name;
            this.value = value;
        }

        public T Value
        {
            get { return value; }
            set { this.value = value; }
        }

        #region INameValueCombination Members

        public string Name
        {
            get { return name; }
        }

        object INameValueCombination.Value
        {
            get { return value; }
            set { this.value = (T) value; }
        }

        #endregion
    }
}